@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Adjuster.cxx,v 1.5.2.2 2000/06/05 21:20:47 mike Exp $"
+// "$Id: Fl_Adjuster.cxx 4609 2005-10-30 17:42:21Z mike $"
 //
 // Adjuster widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 
@@ -54,13 +56,14 @@ void Fl_Adjuster::draw() {
   if (active_r())
     fl_color(selection_color());
   else
-    fl_color(inactive(selection_color()));
+    fl_color(fl_inactive(selection_color()));
   fastarrow.draw(x()+(W-fastarrow_width)/2,
 		 y()+2*dy+(H-fastarrow_height)/2, W, H);
   mediumarrow.draw(x()+dx+(W-mediumarrow_width)/2,
 		   y()+dy+(H-mediumarrow_height)/2, W, H);
   slowarrow.draw(x()+2*dx+(W-slowarrow_width)/2,
 		 y()+(H-slowarrow_width)/2, W, H);
+  if (Fl::focus() == this) draw_focus();
 }
 
 int Fl_Adjuster::handle(int event) {
@@ -68,67 +71,101 @@ int Fl_Adjuster::handle(int event) {
   int delta;
   int mx = Fl::event_x();
   switch (event) {
-  case FL_PUSH:
-    ix = mx;
-    if (w()>=h())
-      drag = 3*(mx-x())/w() + 1;
-    else
-      drag = 3-3*(Fl::event_y()-y()-1)/h();
-    handle_push();
-    redraw();
-    return 1;
-  case FL_DRAG:
-    if (w() >= h()) {
-      delta = x()+(drag-1)*w()/3;	// left edge of button
-      if (mx < delta)
-	delta = mx-delta;
-      else if (mx > (delta+w()/3)) // right edge of button
-	delta = mx-delta-w()/3;
+    case FL_PUSH:
+      if (Fl::visible_focus()) Fl::focus(this);
+      ix = mx;
+      if (w()>=h())
+	drag = 3*(mx-x())/w() + 1;
       else
-	delta = 0;
-    } else {
-      if (mx < x())
-	delta = mx-x();
-      else if (mx > (x()+w()))
-	delta = mx-x()-w();
-      else
-	delta = 0;
-    }
-    switch (drag) {
-    case 3: v = increment(previous_value(), delta); break;
-    case 2: v = increment(previous_value(), delta*10); break;
-    default:v = increment(previous_value(), delta*100); break;
-    }
-    handle_drag(soft() ? softclamp(v) : clamp(v));
-    return 1;
-  case FL_RELEASE:
-    if (Fl::event_is_click()) { // detect click but no drag
-      if (Fl::event_state()&0xF0000) delta = -10;
-      else delta = 10;
+	drag = 3-3*(Fl::event_y()-y()-1)/h();
+      handle_push();
+      redraw();
+      return 1;
+    case FL_DRAG:
+      if (w() >= h()) {
+	delta = x()+(drag-1)*w()/3;	// left edge of button
+	if (mx < delta)
+	  delta = mx-delta;
+	else if (mx > (delta+w()/3)) // right edge of button
+	  delta = mx-delta-w()/3;
+	else
+	  delta = 0;
+      } else {
+	if (mx < x())
+	  delta = mx-x();
+	else if (mx > (x()+w()))
+	  delta = mx-x()-w();
+	else
+	  delta = 0;
+      }
       switch (drag) {
       case 3: v = increment(previous_value(), delta); break;
       case 2: v = increment(previous_value(), delta*10); break;
       default:v = increment(previous_value(), delta*100); break;
       }
       handle_drag(soft() ? softclamp(v) : clamp(v));
-    }
-    drag = 0;
-    redraw();
-    handle_release();
-    return 1;
+      return 1;
+    case FL_RELEASE:
+      if (Fl::event_is_click()) { // detect click but no drag
+	if (Fl::event_state()&0xF0000) delta = -10;
+	else delta = 10;
+	switch (drag) {
+	case 3: v = increment(previous_value(), delta); break;
+	case 2: v = increment(previous_value(), delta*10); break;
+	default:v = increment(previous_value(), delta*100); break;
+	}
+	handle_drag(soft() ? softclamp(v) : clamp(v));
+      }
+      drag = 0;
+      redraw();
+      handle_release();
+      return 1;
+    case FL_KEYBOARD :
+      switch (Fl::event_key()) {
+	case FL_Up:
+          if (w() > h()) return 0;
+	  handle_drag(clamp(increment(value(),-1)));
+	  return 1;
+	case FL_Down:
+          if (w() > h()) return 0;
+	  handle_drag(clamp(increment(value(),1)));
+	  return 1;
+	case FL_Left:
+          if (w() < h()) return 0;
+	  handle_drag(clamp(increment(value(),-1)));
+	  return 1;
+	case FL_Right:
+          if (w() < h()) return 0;
+	  handle_drag(clamp(increment(value(),1)));
+	  return 1;
+	default:
+          return 0;
+      }
+      // break not required because of switch...
+
+    case FL_FOCUS:
+    case FL_UNFOCUS:
+      if (Fl::visible_focus()) {
+        redraw();
+        return 1;
+      } else return 0;
+
+    case FL_ENTER :
+    case FL_LEAVE :
+      return 1;
   }
   return 0;
 }
 
-Fl_Adjuster::Fl_Adjuster(int x, int y, int w, int h, const char* l)
-  : Fl_Valuator(x, y, w, h, l) {
+Fl_Adjuster::Fl_Adjuster(int X, int Y, int W, int H, const char* l)
+  : Fl_Valuator(X, Y, W, H, l) {
   box(FL_UP_BOX);
   step(1, 10000);
-  selection_color(FL_BLACK);
+  selection_color(FL_SELECTION_COLOR);
   drag = 0;
   soft_ = 1;
 }
 
 //
-// End of "$Id: Fl_Adjuster.cxx,v 1.5.2.2 2000/06/05 21:20:47 mike Exp $".
+// End of "$Id: Fl_Adjuster.cxx 4609 2005-10-30 17:42:21Z mike $".
 //
