@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Browser_.cxx,v 1.10.2.14 2000/06/30 04:23:13 spitzak Exp $"
+// "$Id: Fl_Browser_.cxx 4729 2005-12-31 18:26:01Z matt $"
 //
 // Base Browser widget class for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,9 +20,14 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
+#define DISPLAY_SEARCH_BOTH_WAYS_AT_ONCE
+
+#include <stdio.h>
 #include <FL/Fl.H>
 #include <FL/Fl_Widget.H>
 #include <FL/Fl_Browser_.H>
@@ -116,9 +121,9 @@ void Fl_Browser_::update_top() {
   if (position_ != real_position_) {
     void* l;
     int ly;
-    int y = position_;
+    int yy = position_;
     // start from either head or current position, whichever is closer:
-    if (!top_ || y <= (real_position_/2)) {
+    if (!top_ || yy <= (real_position_/2)) {
       l = item_first();
       ly = 0;
     } else {
@@ -130,35 +135,35 @@ void Fl_Browser_::update_top() {
       offset_ = 0;
       real_position_ = 0;
     } else {
-      int h = item_quick_height(l);
+      int hh = item_quick_height(l);
       // step through list until we find line containing this point:
-      while (ly > y) {
+      while (ly > yy) {
 	void* l1 = item_prev(l);
 	if (!l1) {ly = 0; break;} // hit the top
-	l = l1;
-	h = item_quick_height(l);
-	ly -= h;
+	l  = l1;
+	hh = item_quick_height(l);
+	ly -= hh;
       }
-      while ((ly+h) <= y) {
+      while ((ly+hh) <= yy) {
 	void* l1 = item_next(l);
-	if (!l1) {y = ly+h-1; break;}
+	if (!l1) {yy = ly+hh-1; break;}
 	l = l1;
-	ly += h;
-	h = item_quick_height(l);
+	ly += hh;
+	hh = item_quick_height(l);
       }
       // top item must *really* be visible, use slow height:
       for (;;) {
-	h = item_height(l);
-	if ((ly+h) > y) break; // it is big enough to see
+	hh = item_height(l);
+	if ((ly+hh) > yy) break; // it is big enough to see
 	// go up to top of previous item:
 	void* l1 = item_prev(l);
-	if (!l1) {ly = y = 0; break;} // hit the top
-	l = l1; y = position_ = ly = ly-item_quick_height(l);
+	if (!l1) {ly = yy = 0; break;} // hit the top
+	l = l1; yy = position_ = ly = ly-item_quick_height(l);
       }
       // use it:
       top_ = l;
-      offset_ = y-ly;
-      real_position_ = y;
+      offset_ = yy-ly;
+      real_position_ = yy;
     }
     damage(FL_DAMAGE_SCROLL);
   }
@@ -166,47 +171,88 @@ void Fl_Browser_::update_top() {
 
 // Change position(), top() will update when update_top() is called
 // (probably by draw() or handle()):
-void Fl_Browser_::position(int y) {
-  if (y < 0) y = 0;
-  if (y == position_) return;
-  position_ = y;
-  if (y != real_position_) redraw_lines();
+void Fl_Browser_::position(int yy) {
+  if (yy < 0) yy = 0;
+  if (yy == position_) return;
+  position_ = yy;
+  if (yy != real_position_) redraw_lines();
 }
 
-void Fl_Browser_::hposition(int x) {
-  if (x < 0) x = 0;
-  if (x == hposition_) return;
-  hposition_ = x;
-  if (x != real_hposition_) redraw_lines();
+void Fl_Browser_::hposition(int xx) {
+  if (xx < 0) xx = 0;
+  if (xx == hposition_) return;
+  hposition_ = xx;
+  if (xx != real_hposition_) redraw_lines();
 }
 
 // Tell whether item is currently displayed:
-int Fl_Browser_::displayed(void* x) const {
+int Fl_Browser_::displayed(void* p) const {
   int X, Y, W, H; bbox(X, Y, W, H);
   int yy = H+offset_;
   for (void* l = top_; l && yy > 0; l = item_next(l)) {
-    if (l == x) return 1;
+    if (l == p) return 1;
     yy -= item_height(l);
   }
   return 0;
 }
 
-// Insure this item is displayed:
+// Ensure this item is displayed:
 // Messy because we have no idea if it is before top or after bottom:
-void Fl_Browser_::display(void* x) {
+void Fl_Browser_::display(void* p) {
+
+  // First special case - want to display first item in the list?
   update_top();
-  if (x == item_first()) {position(0); return;}
-  int X, Y, W, H; bbox(X, Y, W, H);
+  if (p == item_first()) {position(0); return;}
+
+  int X, Y, W, H, Yp; bbox(X, Y, W, H);
   void* l = top_;
-  Y = -offset_;
-  // see if it is at the top or just above it:
-  if (l == x) {position(real_position_+Y); return;} // scroll up a bit
+  Y = Yp = -offset_;
+  int h1;
+
+  // 2nd special case - want to display item already displayed at top of browser?
+  if (l == p) {position(real_position_+Y); return;} // scroll up a bit
+
+  // 3rd special case - want to display item just above top of browser?
   void* lp = item_prev(l);
-  if (lp == x) {position(real_position_+Y-item_quick_height(lp)); return;}
+  if (lp == p) {position(real_position_+Y-item_quick_height(lp)); return;}
+
+#ifdef DISPLAY_SEARCH_BOTH_WAYS_AT_ONCE
+  // search for item.  We search both up and down the list at the same time,
+  // this evens up the execution time for the two cases - the old way was
+  // much slower for going up than for going down.
+  while (l || lp) {
+    if (l) {
+      h1 = item_quick_height(l);
+      if (l == p) {
+	if (Y <= H) { // it is visible or right at bottom
+	  Y = Y+h1-H; // find where bottom edge is
+	  if (Y > 0) position(real_position_+Y); // scroll down a bit
+	} else {
+	  position(real_position_+Y-(H-h1)/2); // center it
+	}
+	return;
+      }
+      Y += h1;
+      l = item_next(l);
+    }
+    if (lp) {
+      h1 = item_quick_height(lp);
+      Yp -= h1;
+      if (lp == p) {
+	if ((Yp + h1) >= 0) position(real_position_+Yp);
+	else position(real_position_+Yp-(H-h1)/2);
+	return;
+      }
+      lp = item_prev(lp);
+    }
+  }
+#else
+  // Old version went forwards and then backwards:
   // search forward for it:
+  l = top_;
   for (; l; l = item_next(l)) {
-    int h1 = item_quick_height(l);
-    if (l == x) {
+    h1 = item_quick_height(l);
+    if (l == p) {
       if (Y <= H) { // it is visible or right at bottom
 	Y = Y+h1-H; // find where bottom edge is
 	if (Y > 0) position(real_position_+Y); // scroll down a bit
@@ -221,32 +267,32 @@ void Fl_Browser_::display(void* x) {
   l = lp;
   Y = -offset_;
   for (; l; l = item_prev(l)) {
-    int h1 = item_quick_height(l);
+    h1 = item_quick_height(l);
     Y -= h1;
-    if (l == x) {
+    if (l == p) {
       if ((Y + h1) >= 0) position(real_position_+Y);
       else position(real_position_+Y-(H-h1)/2);
       return;
     }
   }
+#endif
 }
 
 // redraw, has side effect of updating top and setting scrollbar:
 
 void Fl_Browser_::draw() {
   int drawsquare = 0;
-  if (damage() & FL_DAMAGE_ALL) { // redraw the box if full redraw
-    Fl_Boxtype b = box() ? box() : FL_DOWN_BOX;
-    draw_box(b, x(), y(), w(), h(), color());
-    drawsquare = 1;
-  }
-
   update_top();
   int full_width_ = full_width();
   int full_height_ = full_height();
   int X, Y, W, H; bbox(X, Y, W, H);
   int dont_repeat = 0;
 J1:
+  if (damage() & FL_DAMAGE_ALL) { // redraw the box if full redraw
+    Fl_Boxtype b = box() ? box() : FL_DOWN_BOX;
+    draw_box(b, x(), y(), w(), h(), color());
+    drawsquare = 1;
+  }
   // see if scrollbar needs to be switched on/off:
   if ((has_scrollbar_ & VERTICAL) && (
 	(has_scrollbar_ & ALWAYS_ON) || position_ || full_height_ > H)) {
@@ -259,7 +305,7 @@ J1:
     top_ = item_first(); real_position_ = offset_ = 0;
     if (scrollbar.visible()) {
       scrollbar.clear_visible();
-      clear_damage(damage()|FL_DAMAGE_SCROLL);
+      clear_damage((uchar)(damage()|FL_DAMAGE_SCROLL));
     }
   }
 
@@ -274,7 +320,7 @@ J1:
     real_hposition_ = 0;
     if (hscrollbar.visible()) {
       hscrollbar.clear_visible();
-      clear_damage(damage()|FL_DAMAGE_SCROLL);
+      clear_damage((uchar)(damage()|FL_DAMAGE_SCROLL));
     }
   }
 
@@ -292,7 +338,7 @@ J1:
     top_ = item_first(); real_position_ = offset_ = 0;
     if (scrollbar.visible()) {
       scrollbar.clear_visible();
-      clear_damage(damage()|FL_DAMAGE_SCROLL);
+      clear_damage((uchar)(damage()|FL_DAMAGE_SCROLL));
     }
   }
 
@@ -308,26 +354,28 @@ J1:
     if (hh <= 0) continue;
     if ((damage()&(FL_DAMAGE_SCROLL|FL_DAMAGE_ALL)) || l == redraw1 || l == redraw2) {
       if (item_selected(l)) {
-	fl_color(active_r() ? selection_color() : inactive(selection_color()));
+	fl_color(active_r() ? selection_color() : fl_inactive(selection_color()));
 	fl_rectf(X, yy+Y, W, hh);
       } else if (!(damage()&FL_DAMAGE_ALL)) {
-	fl_color(color());
-	fl_rectf(X, yy+Y, W, hh);
+	fl_push_clip(X, yy+Y, W, hh);
+	draw_box(box() ? box() : FL_DOWN_BOX, x(), y(), w(), h(), color());
+	fl_pop_clip();
       }
       item_draw(l, X-hposition_, yy+Y, W+hposition_, hh);
-      if (l == selection_) {
-	fl_color(active_r() ? textcolor() : inactive(textcolor()));
-	fl_rect(X+1, yy+Y, W-2, hh);
+      if (l == selection_ && Fl::focus() == this) {
+	draw_box(FL_BORDER_FRAME, X, yy+Y, W, hh, color());
+	draw_focus(FL_NO_BOX, X, yy+Y, W+1, hh+1);
       }
-      int w = item_width(l);
-      if (w > max_width) {max_width = w; max_width_item = l;}
+      int ww = item_width(l);
+      if (ww > max_width) {max_width = ww; max_width_item = l;}
     }
     yy += hh;
   }
   // erase the area below last line:
   if (!(damage()&FL_DAMAGE_ALL) && yy < H) {
-    fl_color(color());
-    fl_rectf(X, yy+Y, W, H-yy);
+    fl_push_clip(X, yy+Y, W, H-yy);
+    draw_box(box() ? box() : FL_DOWN_BOX, x(), y(), w(), h(), color());
+    fl_pop_clip();
   }
   fl_pop_clip();
   redraw1 = redraw2 = 0;
@@ -340,15 +388,15 @@ J1:
     full_width_ = full_width();
     if ((has_scrollbar_ & VERTICAL) &&
 	((has_scrollbar_ & ALWAYS_ON) || position_ || full_height_>H)) {
-      if (!scrollbar.visible()) goto J1;
+      if (!scrollbar.visible()) { damage(FL_DAMAGE_ALL); goto J1; }
     } else {
-      if (scrollbar.visible()) goto J1;
+      if (scrollbar.visible()) { damage(FL_DAMAGE_ALL); goto J1; }
     }
     if ((has_scrollbar_ & HORIZONTAL) &&
 	((has_scrollbar_ & ALWAYS_ON) || hposition_ || full_width_>W)) {
-      if (!hscrollbar.visible()) goto J1;
+      if (!hscrollbar.visible()) { damage(FL_DAMAGE_ALL); goto J1; }
     } else {
-      if (hscrollbar.visible()) goto J1;
+      if (hscrollbar.visible()) { damage(FL_DAMAGE_ALL); goto J1; }
     }
   }
 
@@ -408,6 +456,7 @@ void Fl_Browser_::deleting(void* l) {
   } else {
     // we don't know where this item is, recalculate top...
     real_position_ = 0;
+    offset_ = 0;
     top_ = 0;
   }
   if (l == selection_) selection_ = 0;
@@ -464,8 +513,10 @@ int Fl_Browser_::select(void* l, int i, int docallbacks) {
       display(l);
     }
   }	    
-  Fl::event_clicks(0);
-  if (docallbacks) do_callback();
+  if (docallbacks) {
+    set_changed();
+    do_callback();
+  }
   return 1;
 }
 
@@ -497,10 +548,9 @@ int Fl_Browser_::select_only(void* l, int docallbacks) {
 }
 
 int Fl_Browser_::handle(int event) {
-
   // must do shortcuts first or the scrollbar will get them...
-  if ((event == FL_SHORTCUT || event == FL_KEYBOARD)
-      && type() >= FL_HOLD_BROWSER) {
+  if (event == FL_ENTER || event == FL_LEAVE) return 1;
+  if (event == FL_KEYBOARD && type() >= FL_HOLD_BROWSER) {
     void* l1 = selection_;
     void* l = l1; if (!l) l = top_; if (!l) l = item_first();
     if (l) {
@@ -515,6 +565,7 @@ int Fl_Browser_::handle(int event) {
 	return 1;
       } else switch (Fl::event_key()) {
       case FL_Enter:
+      case FL_KP_Enter:
 	select_only(l, 1);
 	return 1;
       case ' ':
@@ -547,53 +598,87 @@ int Fl_Browser_::handle(int event) {
   if (Fl_Group::handle(event)) return 1;
   int X, Y, W, H; bbox(X, Y, W, H);
   int my;
+// NOTE:
+// instead of:
+//     change = select_only(find_item(my), when() & FL_WHEN_CHANGED)
+// we use the construct:
+//     change = select_only(find_item(my), 0);
+//     if (change && (when() & FL_WHEN_CHANGED)) {
+//	 set_changed();
+//       do_callback();
+//     }
+// See str #834
+// The first form calls the callback *before* setting change.
+// The callback may execute an Fl::wait(), resulting in another
+// call of Fl_Browser_::handle() for the same widget. The sequence
+// of events can be an FL_PUSH followed by an FL_RELEASE.
+// This second call of Fl_Browser_::handle() may result in a -
+// somewhat unexpected - second concurrent invocation of the callback.
+
   static char change;
   static char whichway;
   static int py;
   switch (event) {
   case FL_PUSH:
     if (!Fl::event_inside(X, Y, W, H)) return 0;
+    if (Fl::visible_focus()) {
+      Fl::focus(this);
+      redraw();
+    }
     my = py = Fl::event_y();
     change = 0;
     if (type() == FL_NORMAL_BROWSER || !top_)
       ;
     else if (type() != FL_MULTI_BROWSER) {
-      change = select_only(find_item(my), when() & FL_WHEN_CHANGED);
+      change = select_only(find_item(my), 0);
+      if (change && (when() & FL_WHEN_CHANGED)) {
+	set_changed();
+	do_callback();
+      }
     } else {
       void* l = find_item(my);
       whichway = 1;
       if (Fl::event_state(FL_CTRL)) { // toggle selection:
+      TOGGLE:
 	if (l) {
 	  whichway = !item_selected(l);
-	  change = select(l, whichway, when() & FL_WHEN_CHANGED);
+	  change = select(l, whichway, 0);
+	  if (change && (when() & FL_WHEN_CHANGED)) {
+	    set_changed();
+	    do_callback();
+	  }
 	}
       } else if (Fl::event_state(FL_SHIFT)) { // extend selection:
-	change = 0;
-	if (selection_ && l != selection_) {
-	  void *m = l;
-	  int down = 0;
-	  whichway = item_selected(selection_);
-	  for (m = selection_; m; m = item_next(m)) {
-	    if (m == l) down = 1;
-	  }
-	  void *n;
-	  if (down) {
-	    m = selection_;
-	    n = l;
-	  } else {
-	    m = l;
-	    n = selection_;
-	  }
-	  while (m != n) {
+	if (l == selection_) goto TOGGLE;
+	// state of previous selection determines new value:
+	whichway = l ? !item_selected(l) : 1;
+	// see which of the new item or previous selection is earlier,
+	// by searching from the previous forward for this one:
+	int down;
+	if (!l) down = 1;
+	else {for (void* m = selection_; ; m = item_next(m)) {
+	  if (m == l) {down = 1; break;}
+	  if (!m) {down = 0; break;}
+	}}
+	if (down) {
+	  for (void* m = selection_; m != l; m = item_next(m))
 	    select(m, whichway, when() & FL_WHEN_CHANGED);
-	    m = item_next(m);
+	} else {
+	  void* e = selection_;
+	  for (void* m = item_next(l); m; m = item_next(m)) {
+	    select(m, whichway, when() & FL_WHEN_CHANGED);
+	    if (m == e) break;
 	  }
-	  select(n, whichway, when() & FL_WHEN_CHANGED);
-	  select(l, whichway, when() & FL_WHEN_CHANGED);
-	  change = 1;
 	}
+	// do the clicked item last so the select box is around it:
+	change = 1;
+	if (l) select(l, whichway, when() & FL_WHEN_CHANGED);
       } else { // select only this item
-	change = select_only(l, when() & FL_WHEN_CHANGED);
+	change = select_only(l, 0);
+	if (change && (when() & FL_WHEN_CHANGED)) {
+	  set_changed();
+	  do_callback();
+	}
       }
     }
     return 1;
@@ -606,7 +691,7 @@ int Fl_Browser_::handle(int event) {
       position(p);
     } else if (my > (Y+H) && my > py) {
       int p = real_position_+my-(Y+H);
-      int h = full_height()-H; if (p > h) p = h;
+      int hh = full_height()-H; if (p > hh) p = hh;
       if (p<0) p = 0;
       position(p);
     }
@@ -622,16 +707,23 @@ int Fl_Browser_::handle(int event) {
 	t = l;
 	b = selection_;
       }
-      for (; t && t != b; t = item_next(t))
-	change |= select(t, whichway, when() & FL_WHEN_CHANGED);
+      for (; t && t != b; t = item_next(t)) {
+	char change_t;
+	change_t = select(t, whichway, 0);
+	change |= change_t;
+	if (change_t && (when() & FL_WHEN_CHANGED)) {
+	  set_changed();
+	  do_callback();
+	}
+      }
       if (l) selection_ = l;
     } else {
       void* l1 = selection_;
       void* l =
 	(Fl::event_x()<x() || Fl::event_x()>x()+w()) ? selection_ :
 	find_item(my);
-      select_only(l, when() & FL_WHEN_CHANGED);
       change = (l != l1);
+      select_only(l, when() & FL_WHEN_CHANGED);
     }
     py = my;
     return 1;
@@ -640,19 +732,25 @@ int Fl_Browser_::handle(int event) {
       void* t = selection_; deselect(); selection_ = t;
     }
     if (change) {
+      set_changed();
       if (when() & FL_WHEN_RELEASE) do_callback();
-      else if (!(when()&FL_WHEN_CHANGED)) set_changed();
     } else {
       if (when() & FL_WHEN_NOT_CHANGED) do_callback();
     }
     return 1;
+  case FL_FOCUS:
+  case FL_UNFOCUS:
+    if (type() >= FL_HOLD_BROWSER && Fl::visible_focus()) {
+      redraw();
+      return 1;
+    } else return 0;
   }
 
   return 0;
 }
 
-Fl_Browser_::Fl_Browser_(int x, int y, int w, int h, const char* l)
-  : Fl_Group(x, y, w, h, l),
+Fl_Browser_::Fl_Browser_(int X, int Y, int W, int H, const char* l)
+  : Fl_Group(X, Y, W, H, l),
     scrollbar(0, 0, 0, 0, 0), // they will be resized by draw()
     hscrollbar(0, 0, 0, 0, 0)
 {
@@ -664,15 +762,14 @@ Fl_Browser_::Fl_Browser_(int x, int y, int w, int h, const char* l)
   top_ = 0;
   when(FL_WHEN_RELEASE_ALWAYS);
   selection_ = 0;
-  color(FL_WHITE);
-  selection_color(FL_SELECTION_COLOR);
+  color(FL_BACKGROUND2_COLOR, FL_SELECTION_COLOR);
   scrollbar.callback(scrollbar_callback);
 //scrollbar.align(FL_ALIGN_LEFT|FL_ALIGN_BOTTOM); // back compatability?
   hscrollbar.callback(hscrollbar_callback);
   hscrollbar.type(FL_HORIZONTAL);
   textfont_ = FL_HELVETICA;
-  textsize_ = FL_NORMAL_SIZE;
-  textcolor_ = FL_BLACK;
+  textsize_ = (uchar)FL_NORMAL_SIZE;
+  textcolor_ = FL_FOREGROUND_COLOR;
   has_scrollbar_ = BOTH;
   max_width = 0;
   max_width_item = 0;
@@ -706,5 +803,5 @@ void Fl_Browser_::item_select(void*, int) {}
 int Fl_Browser_::item_selected(void* l) const {return l==selection_;}
 
 //
-// End of "$Id: Fl_Browser_.cxx,v 1.10.2.14 2000/06/30 04:23:13 spitzak Exp $".
+// End of "$Id: Fl_Browser_.cxx 4729 2005-12-31 18:26:01Z matt $".
 //
