@@ -1,7 +1,7 @@
 //
-// "$Id: Fl_Text_Editor.cxx,v 1.9.2.14 2003/01/30 21:42:50 easysw Exp $"
+// "$Id: Fl_Text_Editor.cxx 4288 2005-04-16 00:13:17Z mike $"
 //
-// Copyright 2001-2003 by Bill Spitzak and others.
+// Copyright 2001-2005 by Bill Spitzak and others.
 // Original code Copyright Mark Edel.  Permission to distribute under
 // the LGPL for the FLTK library granted by Mark Edel.
 //
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 #include <stdio.h>
@@ -47,8 +49,6 @@ Fl_Text_Editor::Fl_Text_Editor(int X, int Y, int W, int H,  const char* l)
 
 Fl_Text_Editor::Key_Binding* Fl_Text_Editor::global_key_bindings = 0;
 
-static int ctrl_a(int, Fl_Text_Editor* e);
-
 // These are the default key bindings every widget should start with
 static struct {
   int key;
@@ -102,7 +102,7 @@ static struct {
   { FL_Insert,    FL_CTRL,                  Fl_Text_Editor::kf_copy       },
   { 'v',          FL_CTRL,                  Fl_Text_Editor::kf_paste      },
   { FL_Insert,    FL_SHIFT,                 Fl_Text_Editor::kf_paste      },
-  { 'a',          FL_CTRL,                  ctrl_a                        },
+  { 'a',          FL_CTRL,                  Fl_Text_Editor::kf_select_all },
 
 #ifdef __APPLE__
   // Define CMD+key accelerators...
@@ -110,7 +110,7 @@ static struct {
   { 'x',          FL_COMMAND,               Fl_Text_Editor::kf_cut        },
   { 'c',          FL_COMMAND,               Fl_Text_Editor::kf_copy       },
   { 'v',          FL_COMMAND,               Fl_Text_Editor::kf_paste      },
-  { 'a',          FL_COMMAND,               ctrl_a                        },
+  { 'a',          FL_COMMAND,               Fl_Text_Editor::kf_select_all },
 #endif // __APPLE__
 
   { 0,            0,                        0                             }
@@ -187,7 +187,8 @@ int Fl_Text_Editor::kf_default(int c, Fl_Text_Editor* e) {
   if (e->insert_mode()) e->insert(s);
   else e->overstrike(s);
   e->show_insert_position();
-  if (e->when()&FL_WHEN_CHANGED) e->do_callback(); else e->set_changed();
+  e->set_changed();
+  if (e->when()&FL_WHEN_CHANGED) e->do_callback();
   return 1;
 }
 
@@ -200,7 +201,8 @@ int Fl_Text_Editor::kf_backspace(int, Fl_Text_Editor* e) {
     e->buffer()->select(e->insert_position(), e->insert_position()+1);
   kill_selection(e);
   e->show_insert_position();
-  if (e->when()&FL_WHEN_CHANGED) e->do_callback(); else e->set_changed();
+  e->set_changed();
+  if (e->when()&FL_WHEN_CHANGED) e->do_callback();
   return 1;
 }
 
@@ -208,7 +210,8 @@ int Fl_Text_Editor::kf_enter(int, Fl_Text_Editor* e) {
   kill_selection(e);
   e->insert("\n");
   e->show_insert_position();
-  if (e->when()&FL_WHEN_CHANGED) e->do_callback(); else e->set_changed();
+  e->set_changed();
+  if (e->when()&FL_WHEN_CHANGED) e->do_callback();
   return 1;
 }
 
@@ -266,9 +269,11 @@ int Fl_Text_Editor::kf_ctrl_move(int c, Fl_Text_Editor* e) {
   switch (c) {
     case FL_Home:
       e->insert_position(0);
+      e->scroll(0, 0);
       break;
     case FL_End:
       e->insert_position(e->buffer()->length());
+      e->scroll(e->count_lines(0, e->buffer()->length(), 1), 0);
       break;
     case FL_Left:
       e->previous_word();
@@ -298,20 +303,6 @@ int Fl_Text_Editor::kf_c_s_move(int c, Fl_Text_Editor* e) {
   return 1;
 }
 
-static int ctrl_a(int, Fl_Text_Editor* e) {
-  // make 2+ ^A's in a row toggle select-all:
-  int i = e->buffer()->line_start(e->insert_position());
-  if (i != e->insert_position())
-    return Fl_Text_Editor::kf_move(FL_Home, e);
-  else {
-    if (e->buffer()->selected())
-      e->buffer()->unselect();
-    else
-      Fl_Text_Editor::kf_select_all(0, e);
-  }
-  return 1;
-}
-
 int Fl_Text_Editor::kf_home(int, Fl_Text_Editor* e) {
     return kf_move(FL_Home, e);
 }
@@ -355,7 +346,8 @@ int Fl_Text_Editor::kf_delete(int, Fl_Text_Editor* e) {
     e->buffer()->select(e->insert_position(), e->insert_position()+1);
   kill_selection(e);
   e->show_insert_position();
-  if (e->when()&FL_WHEN_CHANGED) e->do_callback(); else e->set_changed();
+  e->set_changed();
+  if (e->when()&FL_WHEN_CHANGED) e->do_callback();
   return 1;
 }
 
@@ -371,7 +363,8 @@ int Fl_Text_Editor::kf_copy(int, Fl_Text_Editor* e) {
 int Fl_Text_Editor::kf_cut(int c, Fl_Text_Editor* e) {
   kf_copy(c, e);
   kill_selection(e);
-  if (e->when()&FL_WHEN_CHANGED) e->do_callback(); else e->set_changed();
+  e->set_changed();
+  if (e->when()&FL_WHEN_CHANGED) e->do_callback();
   return 1;
 }
 
@@ -379,7 +372,8 @@ int Fl_Text_Editor::kf_paste(int, Fl_Text_Editor* e) {
   kill_selection(e);
   Fl::paste(*e, 1);
   e->show_insert_position();
-  if (e->when()&FL_WHEN_CHANGED) e->do_callback(); else e->set_changed();
+  e->set_changed();
+  if (e->when()&FL_WHEN_CHANGED) e->do_callback();
   return 1;
 }
 
@@ -394,13 +388,13 @@ int Fl_Text_Editor::kf_undo(int , Fl_Text_Editor* e) {
   int ret = e->buffer()->undo(&crsr);
   e->insert_position(crsr);
   e->show_insert_position();
-  if (e->when()&FL_WHEN_CHANGED) e->do_callback(); else e->set_changed();
+  e->set_changed();
+  if (e->when()&FL_WHEN_CHANGED) e->do_callback();
   return ret;
 }
 
 int Fl_Text_Editor::handle_key() {
-
-  // Call fltk's rules to try to turn this into a printing character.
+  // Call FLTK's rules to try to turn this into a printing character.
   // This uses the right-hand ctrl key as a "compose prefix" and returns
   // the changes that should be made to the text, as a number of
   // bytes to delete and a string to insert:
@@ -413,6 +407,8 @@ int Fl_Text_Editor::handle_key() {
       else overstrike(Fl::event_text());
     }
     show_insert_position();
+    set_changed();
+    if (when()&FL_WHEN_CHANGED) do_callback();
     return 1;
   }
 
@@ -421,15 +417,15 @@ int Fl_Text_Editor::handle_key() {
   Key_Func f;
   f = bound_key_function(key, state, global_key_bindings);
   if (!f) f = bound_key_function(key, state, key_bindings);
-
   if (f) return f(key, this);
   if (default_key_function_ && !state) return default_key_function_(c, this);
   return 0;
 }
 
 void Fl_Text_Editor::maybe_do_callback() {
-  if (changed() || (when()&FL_WHEN_NOT_CHANGED)) {
-    clear_changed(); do_callback();}
+//  printf("Fl_Text_Editor::maybe_do_callback()\n");
+//  printf("changed()=%d, when()=%x\n", changed(), when());
+  if (changed() || (when()&FL_WHEN_NOT_CHANGED)) do_callback();
 }
 
 int Fl_Text_Editor::handle(int event) {
@@ -439,18 +435,21 @@ int Fl_Text_Editor::handle(int event) {
     dragType = -1;
     Fl::paste(*this, 0);
     Fl::focus(this);
-    if (when()&FL_WHEN_CHANGED) do_callback(); else set_changed();
+    set_changed();
+    if (when()&FL_WHEN_CHANGED) do_callback();
     return 1;
   }
 
   switch (event) {
     case FL_FOCUS:
       show_cursor(mCursorOn); // redraws the cursor
+      if (buffer()->selected()) redraw(); // Redraw selections...
       Fl::focus(this);
       return 1;
 
     case FL_UNFOCUS:
       show_cursor(mCursorOn); // redraws the cursor
+      if (buffer()->selected()) redraw(); // Redraw selections...
     case FL_HIDE:
       if (when() & FL_WHEN_RELEASE) maybe_do_callback();
       return 1;
@@ -467,20 +466,20 @@ int Fl_Text_Editor::handle(int event) {
       if (insert_mode()) insert(Fl::event_text());
       else overstrike(Fl::event_text());
       show_insert_position();
-      if (when()&FL_WHEN_CHANGED) do_callback(); else set_changed();
+      set_changed();
+      if (when()&FL_WHEN_CHANGED) do_callback();
       return 1;
 
-// CET - FIXME - this will clobber the window's current cursor state!
-//    case FL_ENTER:
+    case FL_ENTER:
+// MRS: WIN32 only?  Need to test!
 //    case FL_MOVE:
-//    case FL_LEAVE:
-//      if (Fl::event_inside(text_area)) fl_cursor(FL_CURSOR_INSERT);
-//      else fl_cursor(FL_CURSOR_DEFAULT);
+      show_cursor(mCursorOn);
+      return 1;
   }
 
   return Fl_Text_Display::handle(event);
 }
 
 //
-// End of "$Id: Fl_Text_Editor.cxx,v 1.9.2.14 2003/01/30 21:42:50 easysw Exp $".
+// End of "$Id: Fl_Text_Editor.cxx 4288 2005-04-16 00:13:17Z mike $".
 //
